#ifndef __RESOURCE_flashback_H__
#define __RESOURCE_flashback_H__

#include <gio/gio.h>

extern GResource *flashback_get_resource (void);

extern void flashback_register_resource (void);
extern void flashback_unregister_resource (void);

#endif
